from vcRslProcessHandler import * 
from vcBehaviour import * 
import vcMatrix, vcVector, math 
  
app = getApplication() 
comp = getComponent() 
RAD_TO_DEG = math.pi/180.0 

#-----

class gunControl:
  #Class for weld gun control
  def __init__(self, exe, stat):
    self.Joint = None
    self.InPort = -1
    self.OutPort = -1
    self.InSignal = None
    self.OutSignal = None
    self.IO = False
    
    #Get tool component
    tool_comp = None
    try:
      flange_node = exe.Controller.FlangeNode
      ifaces = flange_node.findBehavioursByType(VC_ONETOONEINTERFACE)
      for iface in ifaces:
        if iface.IsAbstract or not iface.IsConnected:
          continue
        tool_comp = iface.ConnectedComponent
        break
    except:
      pass
    
    #Servo controlled
    try:
      servo = tool_comp.findBehavioursByType(VC_SERVOCONTROLLER)[0]
      joint = servo.Joints[0]
      if joint.ExternalController:
        self.Joint = joint
    except:
      pass
    
    #IO controlled
    if not self.Joint and tool_comp:
      try:
        robot_in_port = tool_comp.OutPort
        robot_out_port = tool_comp.InPort
        robot_in = exe.DigitalInputSignals.getInternalPortSignal(robot_in_port)
        robot_out = exe.DigitalOutputSignals.getInternalPortSignal(robot_out_port)
        if robot_in and robot_out:
          self.InPort = robot_in_port
          self.OutPort = robot_out_port
          self.InSignal = robot_in
          self.OutSignal = robot_out
          self.IO = True
      except:
        pass
  
#-----

def OnStatementExecute(exe, stat):
  prop_spot = stat.getProperty('SpotFrame')
  prop_approach = stat.getProperty('Approach')
  prop_approach_axis = stat.getProperty('ApproachAxis')
  prop_time = stat.getProperty('WeldTime')
  prop_thickness = stat.getProperty('Thickness')
  
  if stat.Type == VC_STATEMENT_LINMOTION and prop_spot and prop_spot.Value and prop_approach and prop_approach_axis and prop_time and prop_thickness:
    #Execute as spot statement
    ctr = exe.Controller
    ctr.clearTargets() 
    app_tgt = ctr.createTarget()
    tgt = ctr.createTarget()
    stat.writeToTarget(app_tgt)
    stat.writeToTarget(tgt)
    
    #Form approach/departure target
    m = app_tgt.Target
    if prop_approach_axis.Value == 'X+':
      m.translateRel(-prop_approach.Value,0,0)
    elif prop_approach_axis.Value == 'X-':
      m.translateRel(prop_approach.Value,0,0)
    elif prop_approach_axis.Value == 'Y+':
      m.translateRel(0,-prop_approach.Value,0)
    elif prop_approach_axis.Value == 'Y-':
      m.translateRel(0,prop_approach.Value,0)
    elif prop_approach_axis.Value == 'Z+':
      m.translateRel(0,0,-prop_approach.Value,)
    else:#if prop_approach_axis.Value == 'Z-':
      m.translateRel(0,0,prop_approach.Value)
    app_tgt.Target = m
    
    gun = gunControl(exe,stat)
    if gun.Joint:
      #servo gun
      try:
        index = gun.Joint.ExternalController.Joints.index(gun.Joint)
        js = tgt.JointValues
        js[index] = prop_thickness.Value
        tgt.JointValues = js
      except:
        pass
    
    #Move to target via approach point
    ctr.addTarget(app_tgt)
    ctr.addTarget(tgt)
    ctr.move()
    
    if gun.IO:
      #IO controlled gun, open gun and wait for feedback signal
      gun.OutSignal.signal(True)
      #Signal triggers won't work with signal maps ATM
      delay(0.2)
      #triggerCondition(lambda: getTrigger() == gun.InSignal and gun.InSignal.Value == True)
    
    #Wait for weld to complete
    delay(prop_time.Value)
    
    if gun.IO:
      #IO controlled gun, close gun and wait for feedback signal
      gun.OutSignal.signal(False)
      #Signal triggers won't work with signal maps ATM, just wait a bit
      delay(0.2)
      #triggerCondition(lambda: getTrigger() == gun.InSignal and gun.InSignal.Value == True)
      
    #Move to departure point
    ctr.addTarget(app_tgt)
    ctr.move()
  else: 
    #Execute as normal Lin motion
    exe.callStatement(stat) 
    