#-----
#EasySpot - Create spot statements
#
#Create spot statements by clicking positions on the 3D model.
#-----
from __future__ import with_statement
from vcCommand import *
from vcHelpers.Application import *
from vcHelpers.Selection import *
from vcHelpers.Robot2 import *
import os.path, vcMatrix, vcVector, vcSelectionManager

app = getApplication()
sim = getSimulation()
cmd = getCommand()
snap = app.findCommand('interactiveSnap') 
es_cmd = app. findCommand('easyspot')

#-----

def first_state():
  #Create spot statements state 1
  global exe,rou,ctr,basename,toolname,basenames,toolnames,robotconfig,robotconfigs,index,new_stats,mt_cache
  
  new_stats = []
  ctr = None
  
  try:
    exe, rou, ctr, basename, toolname, basenames, toolnames, robotconfig,  robotconfigs = getSelectedRobotsData()
  except:
    print 'No robot selected.'
    return
  
  #Create handler
  createSpotHandler(exe)
  
  #Store current pos and put basic conf to robot
  mt_cache = ctr.createTarget()
  mt_basic = ctr.createTarget()
  mt_basic.RobotConfig = 0
  ctr.moveImmediate(mt_basic)
  
  #Pick targets
  index = getSelectedStatementIndex()
  startPicking()
  
addState( first_state )

#-----

def targetPicked(args): 
  #Create spot statement at selected position
  global exe,rou,ctr,basename,toolname,basenames,toolnames,robotconfig,robotconfigs,snap,index,new_stats
  
  weld_ID = es_cmd.getProperty('Create Spot Statements::Weld ID').Value
  approach = es_cmd.getProperty('Create Spot Statements::Approach').Value
  weld_time = es_cmd.getProperty('Create Spot Statements::Weld Time').Value
  thickness = es_cmd.getProperty('Create Spot Statements::Thickness').Value
  auto_thickness = es_cmd.getProperty('Create Spot Statements::Auto Thickness').Value
  gun_joint_value = es_cmd.getProperty('Create Spot Statements::Gun Joint').Value
  approach_axis = es_cmd.getProperty('Create Spot Statements::TCP Approach Axis').Value
  flip_tcp = es_cmd.getProperty('Create Spot Statements::TCP Position').Value == 'At Flip Side'
  auto_ori = es_cmd.getProperty('Create Spot Statements::Auto Orientate Points').Value
  
  #Get target
  m = vcMatrix.new()
  m.P = snap.TargetPosition
  if snap.TargetMatrixAvailable:
    m = snap.TargetMatrix
  elif snap.TargetNormalAvailable:
    m.N = snap.TargetNormal
    m.uniform()
    m.rotateRelY(90)
  m.uniform()
  #m.A aligned with picked surface normal
  
  #Get thickness
  thickness_this = thickness
  if auto_thickness:
    #Find thickness with ray cast on the other side (20 mm away)
    ray = vcMatrix.new(m)
    ray.translateRel(0,0,-20)
    hit = app.rayCast(ray, 20, ctr.Component, True)
    if hit:
      diff = hit[2].P - m.P
      thickness_this = round(diff.length(),1)
  
  if flip_tcp:
    #Move TCP to other side and flip orientation
    m.translateRel(0,0,-thickness_this)
    m.rotateRelX(180)
  
  #Set approach axis
  if approach_axis == 'X+':
    m.rotateRelY(90)
  elif approach_axis == 'X-':
    m.rotateRelY(-90)
  elif approach_axis == 'Y+':
    m.rotateRelX(-90)
  elif approach_axis == 'Y-':
    m.rotateRelX(90)
  elif approach_axis == 'Z+':
    m.rotateRelX(180)
  else:#if approach_axis == 'Z-':
    pass
  
  #Create new statement at selected index
  if index >= 0:
    stat = rou.addStatement(VC_STATEMENT_LINMOTION, index + 1)
    index = index + 1
  else:
    stat = rou.addStatement(VC_STATEMENT_LINMOTION)
  stat.readIn()
  name = getUniqueSpotName(exe, weld_ID)
  stat.Positions[0].Name = name
  stat.Positions[0].PositionInWorld = m
  
  #Create spot properties
  prop = stat.createProperty(VC_BOOLEAN, 'SpotFrame')
  prop.Value = True
  
  prop = stat.createProperty(VC_REAL, 'Approach')
  prop.Value = approach
  
  prop = stat.createProperty(VC_STRING,'ApproachAxis',VC_PROPERTY_STEP)
  prop.StepValues = ['Z+', 'Z-', 'X+', 'X-', 'Y+', 'Y-']
  prop.Value = approach_axis
  
  prop = stat.createProperty(VC_REAL, 'WeldTime')
  prop.Value = weld_time
  
  prop = stat.createProperty(VC_REAL, 'Thickness')
  prop.Value = thickness_this
  
  setToolJointValue(stat, gun_joint_value)
  
  new_stats.append(stat)
  
  #Continue selecteing spot positions
  app.render()
  snap.execute()
  
#-----
  
def startPicking(): 
  global snap 
  #snap.clear() 
  snap.SnapOnPoint = True 
  snap.SnapOnEdge = True 
  snap.SnapOnPolygon = True 
  snap.SnapOnSurface = True 
  snap.SnapOnNodeCenter = False 
  snap.SnapOnNodeOrigin = False 
  snap.OnTargetSet = targetPicked 
  #snap.OnStop = StopPicking
  print 'Pick spot postions. Press ESC to finish.' 
  snap.execute()
  
#-----

def OnStop():
  global exe,rou,ctr,basename,toolname,basenames,toolnames,robotconfig,robotconfigs,\
  tgts,snap,mt_cache
  
  auto_ori = es_cmd.getProperty('Create Spot Statements::Auto Orientate Points').Value
  
  if ctr and auto_ori:
    #Auto orientation
    rotateThem()
  
  #Restore robot pos
  if ctr:
    ctr.moveImmediate(mt_cache)
    ctr.update()
    app.render()
  
  snap.clear()
  snap.OnTargetSet = None
  
  
  #Keep easyspot GUI alive
  es_cmd.executeInActionPanel()
  
#-----


def getExecutor():
  sel = app.SelctionManager.getSelection()
  exes = comp.findBehavioursByType(VC_ROBOTEXECUTOR)
  if len(exes) >= 1:
    return exes[0]
  return None

#-----

def createSpotHandler(exe):
  #Create spot statement handler
  
  comp = exe.Component
  handler = comp.getBehaviour('SpotHandler')
  if not handler:
    handler = comp.createBehaviour(VC_PYTHONPROCESSHANDLER,'SpotHandler')
  
  folder = os.path.dirname(cmd.getCommandPath())[8:] + '\\'
  file = open (folder + 'SpotHandler.py', 'r')
  handler.Script = file.read()
  file.close() 
  
  exe.StatementHandler = handler
  
#-----

def getUniqueSpotName(exe, weld_ID):
  index = 1
  step = 1
  rous = []
  rous.append(exe.Program.MainRoutine)
  rous.extend(exe.Program.Routines)
  names = []
  for rou in rous:
    for stat in rou.Statements:
      try:
        for pos in stat.Positions:
          names.append(pos.Name)
      except:
        continue
  if not weld_ID in names:
    return weld_ID
  seed_name = getBaseName(weld_ID)
  while True:
    name = seed_name + '_' + str(index)
    if name in names:
      index = index + step
      continue
    else:
      return name

#-----

def getBaseName(name):
  #Return base name e.g. getBaseName('Frame_10') => 'Frame'
  index = name.rfind('_')
  if index >= 0:
    base = name[0:index]
  else:
    base = name
  return base
  
#-----

def getSelectedStatementIndex():
  sel = app.SelectionManager.getSelection(VC_SELECTION_STATEMENT)
  if sel:
    last = sel[-1]
    scope = last.ParentScope
    index = -1
    for i in range(0, len(scope.Statements)):
      if scope.Statements[i] == last:
        index = i
        break
    return index

#-----

def rotateThem():
  #Auto-orientate new spots about approach axis. Check for collision and reachability and select closest candidate
  global ctr,new_stats
  
  if len(new_stats) == 0:
    return
  
  #Get tool component
  eoat = None
  try:
    eoat = new_stats[0].Tool.Node.Component
    if not eoat:
      return
  except:
    return
    
  step_angle = 22.5
  robot_loc = ctr.Component.WorldPositionMatrix
  test_rots = [x*step_angle for x in range(int (360/step_angle) ) ]
  detector = sim.newCollisionDetector()
  detector.NodeListA = [ (eoat, VC_NODELIST_INCLUDE, VC_NODELIST_TREE ),  (ctr.Component, VC_NODELIST_INCLUDE, VC_NODELIST_TREE )]
  detector.NodeListB = [ (sim.World, VC_NODELIST_INCLUDE, VC_NODELIST_TREE ) , (eoat, VC_NODELIST_EXCLUDE, VC_NODELIST_TREE ), (ctr.Component, VC_NODELIST_EXCLUDE, VC_NODELIST_TREE )]
  mt = ctr.createTarget()
  prev_tgt = None
  
  #iterate all new spots
  for stat in new_stats:
    approach_axis = stat.getProperty('ApproachAxis').Value
    clearance = 1
    frame = stat.Positions[0]
    candidates = []
    for a in test_rots:
      m = frame.PositionInReference
      
      #Rotate about approach axis and move tcp a bit away from the work piece
      if approach_axis == 'X+':
        m.rotateRelX(a)
        m.translateRel(-clearance,0,0)
      elif approach_axis == 'X-':
        m.rotateRelX(a)
        m.translateRel(clearance,0,0)
      elif approach_axis == 'Y+':
        m.rotateRelY(a)
        m.translateRel(0,-clearance,0)
      elif approach_axis == 'Y-':
        m.rotateRelY(a)
        m.translateRel(0,clearance,0)
      elif approach_axis == 'Z+':
        m.rotateRelZ(a)
        m.translateRel(0,0,-clearance)
      else:#if approach_axis == 'Z-':
        m.rotateRelZ(a)
        m.translateRel(0,0,clearance)
      
      mt.Target = m
      if mt.getConfigWarning(0) != 0:
        #pass, if first config gives no warnings
        continue
      ctr.moveImmediate(mt)
      app.render()
      hit = detector.testOneCollision(0.1)
      if not hit:
        #No collision, add to candidates
        candidates.append([a, eoat.WorldPositionMatrix, m])
    if candidates:
      if not prev_tgt:
        #Get closest to robot
        closest = min( candidates, key = lambda x: ( x[1].P-robot_loc.P ).length() )
      else:
        #Get closest to previous target orientation
        if not 'X' in approach_axis:
          closest = min( candidates, key = lambda x: ( x[2].N.angle(prev_tgt.N) ) )
        else:
          closest = min( candidates, key = lambda x: ( x[2].A.angle(prev_tgt.A) ) )
      mtx = frame.PositionInWorld
      #
      if approach_axis in ['X+', 'X-']:
        mtx.rotateRelX(closest[0])
      elif approach_axis in ['Y+', 'Y-']:
        mtx.rotateRelY(closest[0])
      else:#if approach_axis in ['Z+', 'Z-']:
        mtx.rotateRelZ(closest[0])
      #
      prev_tgt = mtx
      frame.PositionInWorld = mtx
    else:
      #no candidates use current
      pass
  
#-----

def getToolJoint(stat):
  try:
    node = stat.Tool.Node
    comp = node.Component
    servo = comp.findBehavioursByType(VC_SERVOCONTROLLER)[0]
    joint = servo.Joints[0]
    if joint.ExternalController:
      return joint
    else:
      return None
  except:
    return None
    
#-----
 
def setToolJointValue(stat, gun_joint_value):
  gun_joint = getToolJoint(stat)
  if gun_joint:
    try:
      index = gun_joint.ExternalController.Joints.index(gun_joint)
      js = stat.Positions[0].JointValues
      js[index] = gun_joint_value
      stat.Positions[0].setJoints(js)
    except:
      gun_joint_value = 0
  else:
    gun_joint_value = 0
  return gun_joint_value
  
#-----