#-----
#EasySpot - Export CSV
#
#Export all spot statements into a CSV file.
#-----
from __future__ import with_statement
from vcCommand import *
from vcHelpers.Application import *
from vcHelpers.Selection import *
from vcHelpers.Robot2 import *
import os.path, vcMatrix, vcVector, vcSelectionManager
#
app = getApplication()
sim = getSimulation()
cmd = getCommand()
es_cmd = app.findCommand('easyspot')
  
#-----
  
def first_state():
  global stats
  
  stats = getSpotStatements()
  if len(stats) == 0:
    print 'No spots found.'
    return
  
  save_dialog = app.findCommand('dialogSaveFile')
  save_dialog.OnPostExecute = writeCSV
  save_dialog.Param_3 = 'CSV Files (*.csv)|*.csv'
  save_dialog.Param_4 = 'Choose a csv file'
  save_dialog.execute()
  
addState( first_state )

#-----

def OnStop():
  #Keep easyspot GUI alive
  es_cmd.executeInActionPanel()

#-----

def writeCSV(save_dialog):
  global stats
  
  save_dialog.OnPostExecute = None
  file_ok = save_dialog.Param_2
  if not file_ok:
    print 'Easyspot CSV export cancelled.'
    return
    
  uri = save_dialog.Param_1[8:]
  with open(uri,'w') as csv_file:
    columns = ['RobotName', 'Routine', 'WeldID', 'X', 'Y', 'Z', 'W', 'P', 'R', 'MotionType', 'Base', 'Tool', 'Thickness', 'WeldTime', 'Approach', 'ApproachAxis', 'GunJoint']
    separator = ';'
    line = separator.join([str(x) for x in columns]) + '\n'
    csv_file.write(line)
    
    for stat in stats:
      robot_ID = stat.ParentRoutine.Program.Executor.Component.Name
      rou_ID = stat.ParentRoutine.Name
      weld_ID = stat.Positions[0].Name
      x = '%.6f' % stat.Positions[0].PositionInReference.P.X
      y = '%.6f' % stat.Positions[0].PositionInReference.P.Y
      z = '%.6f' % stat.Positions[0].PositionInReference.P.Z
      w = '%.6f' % stat.Positions[0].PositionInReference.WPR.X
      p = '%.6f' % stat.Positions[0].PositionInReference.WPR.Y
      r = '%.6f' % stat.Positions[0].PositionInReference.WPR.Z
      mot_type = 'LIN'
      base = stat.Base.Name
      tool = stat.Tool.Name
      thickness = '%.3f' % stat.getProperty('Thickness').Value
      weld_time = '%.3f' % stat.getProperty('WeldTime').Value
      approach = '%.3f' % stat.getProperty('Approach').Value
      approach_axis = stat.getProperty('ApproachAxis').Value
      gun_joint_value = '%.3f' % getToolJointValue(stat)
      data = [robot_ID, rou_ID, weld_ID, x, y, z, w, p, r, mot_type, base, tool, thickness, weld_time, approach, approach_axis, gun_joint_value]
      line = separator.join([str(x) for x in data]) + '\n'
      csv_file.write(line)

#-----

def getSpotStatements():
  #Collect all sposts from all robots
  stats = []
  for comp in app.Components:
    exes = comp.findBehavioursByType(VC_ROBOTEXECUTOR)
    for exe in exes:
      for stat in exe.Program.MainRoutine.Statements:
        prop = stat.getProperty('SpotFrame')
        if prop and prop.Value:
          stats.append(stat)
      for rou in exe.Program.Routines:
        for stat in rou.Statements:
          prop = stat.getProperty('SpotFrame')
          if prop and prop.Value:
            stats.append(stat)
  return stats
  
#-----
  
def getToolJoint(stat):
  try:
    node = stat.Tool.Node
    comp = node.Component
    servo = comp.findBehavioursByType(VC_SERVOCONTROLLER)[0]
    joint = servo.Joints[0]
    if joint.ExternalController:
      return joint
    else:
      return None
  except:
    return None
    
#-----
 
def getToolJointValue(stat):
  gun_joint = getToolJoint(stat)
  if gun_joint:
    try:
      index = gun_joint.ExternalController.Joints.index(gun_joint)
      js = stat.Positions[0].JointValues
      gun_joint_value = js[index]
    except:
      gun_joint_value = 0
  else:
    gun_joint_value = 0
  return gun_joint_value