#-----
#EasySpot - Import CSV
#
#Import spot statements from a CSV file. Can also used for mirroring spots.
#-----
from __future__ import with_statement
from vcCommand import *
from vcHelpers.Application import *
from vcHelpers.Selection import *
from vcHelpers.Robot2 import *
import os.path, vcMatrix, vcVector, vcSelectionManager

app = getApplication()
sim = getSimulation()
cmd = getCommand()
es_cmd = app. findCommand('easyspot')
  
#-----
  
def first_state():
  open_dialog = app.findCommand('dialogOpen')
  open_dialog.OnPostExecute = readCSV
  open_dialog.Param_3 = 'CSV Files (*.csv)|*.csv'
  open_dialog.execute()
  
addState( first_state )

#-----

def OnStop():
  #Keep easyspot GUI alive
  es_cmd.executeInActionPanel()

#-----

def readCSV(open_dialog):
  open_dialog.OnPostExecute = None
  uri = open_dialog.Param_1[8:]
  import_options = es_cmd.getProperty('CSV File::Import Duplicate Spots').Value
  import_mirror = es_cmd.getProperty('CSV File::Import Mirroring').Value
  if import_mirror == 'BASE XZ':
    mirror_plane = vcMatrix.new()
    mirror_plane.rotateRelZ(90)
  elif import_mirror == 'BASE XY':
    mirror_plane = vcMatrix.new()
    mirror_plane.rotateRelY(90)
  elif import_mirror == 'BASE YZ':
    mirror_plane = vcMatrix.new()
  else:
    mirror_plane = None
  spots = []
  robot_IDs = []
  with open(uri, 'r') as csv_file:
    separator = ';'
    for line in csv_file:
      data = line.split(separator)
      try:
        robot_ID = data[0]
        rou_ID = data[1]
        weld_ID = data[2]
        x = float(data[3])
        y = float(data[4])
        z = float(data[5])
        w = float(data[6])
        p = float(data[7])
        r = float(data[8])
        mot_type = data[9]
        base_name = data[10]
        tool_name = data[11]
        thickness = float(data[12])
        weld_time = float(data[13])
        approach = float(data[14])
        approach_axis = data[15]
        gun_joint_value = float(data[16])
      except:
        continue
      spots.append([robot_ID, rou_ID, weld_ID, x, y, z, w, p, r, mot_type, base_name, tool_name, thickness, weld_time, approach, approach_axis, gun_joint_value])
      if not robot_ID in robot_IDs:
        robot_IDs.append(robot_ID)
        
  #Clear old robot programs
  if import_options == 'Clear old program':
    for robot_ID in robot_IDs:
      comp = app.findComponent(robot_ID)
      exe = getExecutor(comp)
      if exe:
        clearProgram(exe)
        
  #Create handlers (in case of fresh station)
  for robot_ID in robot_IDs:
    comp = app.findComponent(robot_ID)
    exe = getExecutor(comp)
    if exe:
      createSpotHandler(exe)
        
  #Create/modify spots
  for data in spots:
    robot_ID = data[0]
    rou_ID = data[1]
    weld_ID = data[2]
    x = float(data[3])
    y = float(data[4])
    z = float(data[5])
    w = float(data[6])
    p = float(data[7])
    r = float(data[8])
    mot_type = data[9]
    base_name = data[10]
    tool_name = data[11]
    thickness = float(data[12])
    weld_time = float(data[13])
    approach = float(data[14])
    approach_axis = data[15]
    gun_joint_value = float(data[16])
    
    comp = app.findComponent(robot_ID)
    exe = getExecutor(comp)
    if not exe:
      print 'No robot found for spot: ' + weld_ID + '. Skipping spot.'
      continue
      
    if import_options == 'Rename duplicate spots':
      weld_ID = getUniqueSpotName(exe, weld_ID)
    
    stat = getOldSpotStatement(exe, weld_ID)
    if stat:
      if import_options == 'Skip duplicate spots':
        print 'Skipping duplicate spot: ' + weld_ID + '.'
        continue
    else:
      rou = getRoutine(exe, rou_ID)
      stat = rou.addStatement(VC_STATEMENT_LINMOTION)
      stat.readIn()
      stat.createProperty(VC_BOOLEAN, 'SpotFrame')
      stat.createProperty(VC_REAL, 'Approach')
      prop = stat.createProperty(VC_STRING,'ApproachAxis',VC_PROPERTY_STEP)
      prop.StepValues = ['Z+', 'Z-', 'X+', 'X-', 'Y+', 'Y-']
      stat.createProperty(VC_REAL, 'WeldTime')
      stat.createProperty(VC_REAL, 'Thickness')
      
    m = vcMatrix.new()
    m.translateAbs(x,y,z)
    m.setWPR(w,p,r)
    
    if mirror_plane:
      m = mirrorMatrix(m, mirror_plane)
    
    stat.Positions[0].Name = weld_ID
    stat.Positions[0].PositionInReference = m
    stat.Base = getBase(exe.Controller, base_name)
    stat.Tool = getTool(exe.Controller, tool_name)
    stat.getProperty('SpotFrame').Value = True
    stat.getProperty('Approach').Value = approach
    stat.getProperty('ApproachAxis').Value = approach_axis
    stat.getProperty('WeldTime').Value = weld_time
    stat.getProperty('Thickness').Value = thickness
    setToolJointValue(stat, gun_joint_value)
    
   
  app.render()
  
#-----
    
def createSpotHandler(exe):
  #Create spot statement handler
  
  comp = exe.Component
  handler = comp.getBehaviour('SpotHandler')
  if not handler:
    handler = comp.createBehaviour(VC_PYTHONPROCESSHANDLER,'SpotHandler')
  
  folder = os.path.dirname(cmd.getCommandPath())[8:] + '\\'
  file = open (folder + 'SpotHandler.py', 'r')
  handler.Script = file.read()
  file.close() 
  
  exe.StatementHandler = handler
  
#-----

def getExecutor(comp):
  exes = comp.findBehavioursByType(VC_ROBOTEXECUTOR)
  if len(exes) >= 1:
    return exes[0]
  return None

#-----

def clearProgram(exe):
  exe.Program.MainRoutine.clear()
  rous = exe.Program.Routines
  for rou in rous:
    exe.Program.deleteRoutine(rou)

#-----

def getOldSpotStatement(exe, weld_ID):
  rous = exe.Program.Routines
  rous.insert(0, exe.Program.MainRoutine)
  for rou in rous:
    for stat in rou.Statements:
      if stat.Type != VC_STATEMENT_LINMOTION:
        continue
      prop1 = stat.getProperty('SpotFrame')
      prop2 = stat.getProperty('Approach')
      prop3 = stat.getProperty('WeldTime')
      prop4 = stat.getProperty('Thickness')
      pos = stat.Positions[0]
      if prop1 and prop1.Value and prop2 and prop3 and prop4 and pos.Name == weld_ID:
        return stat
  return None
  
#-----
  
def getRoutine(exe, rou_ID):
  rous = exe.Program.Routines
  rous.insert(0, exe.Program.MainRoutine)
  for rou in rous:
    if rou_ID == rou.Name:
      return rou
  rou = exe.Program.addRoutine(rou_ID)
  return rou
  
#-----

def getBase(ctr, base_name):
  for base in ctr.Bases:
    if base.Name == base_name:
      return base
  return None
    
#-----

def getTool(ctr, tool_name):
  for tool in ctr.Tools:
    if tool.Name == tool_name:
      return tool
  return None
    
#-----
  
def getToolJoint(stat):
  try:
    node = stat.Tool.Node
    comp = node.Component
    servo = comp.findBehavioursByType(VC_SERVOCONTROLLER)[0]
    joint = servo.Joints[0]
    if joint.ExternalController:
      return joint
    else:
      return None
  except:
    return None
    
#-----
 
def setToolJointValue(stat, gun_joint_value):
  gun_joint = getToolJoint(stat)
  if gun_joint:
    try:
      index = gun_joint.ExternalController.Joints.index(gun_joint)
      js = stat.Positions[0].JointValues
      js[index] = gun_joint_value
      stat.Positions[0].setJoints(js)
    except:
      gun_joint_value = 0
  else:
    gun_joint_value = 0
  return gun_joint_value
  
#-----

def getUniqueSpotName(exe, weld_ID):
  index = 1
  step = 1
  rous = []
  rous.append(exe.Program.MainRoutine)
  rous.extend(exe.Program.Routines)
  names = []
  for rou in rous:
    for stat in rou.Statements:
      try:
        for pos in stat.Positions:
          names.append(pos.Name)
      except:
        continue
  if not weld_ID in names:
    return weld_ID
  seed_name = getBaseName(weld_ID)
  while True:
    name = seed_name + '_' + str(index)
    if name in names:
      index = index + step
      continue
    else:
      return name

#-----
      
def getBaseName(name):
  #Return base name e.g. getBaseName('Frame_10') => 'Frame'
  index = name.rfind('_')
  if index >= 0:
    base = name[0:index]
  else:
    base = name
  return base
  
#-----

def mirrorMatrix(m,mirror_plane):
  #Mirror position matrix using mirror_plane (Y and Z span the plane)
  #Target matrix.O is computed form N and A to keep matrix right-handed
  imirror_plane = vcMatrix.new(mirror_plane)
  imirror_plane.invert()
  m_in_plane = imirror_plane * m
  #
  N = m_in_plane.N
  N.X = -N.X
  A = m_in_plane.A
  A.X = -A.X
  O = A ^ N
  O.normalize()
  P = m_in_plane.P
  P.X = -P.X
  #
  m_in_plane.N = N
  m_in_plane.O = O
  m_in_plane.A = A
  m_in_plane.P = P
  mirrored_m = mirror_plane * m_in_plane
  mirrored_m.uniform()
  return mirrored_m
  
#-----